package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Argument;
import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodVerifier;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ImplicitNullAnnotationVerifier {
  org.eclipse.jdt.internal.compiler.lookup.ImplicitNullAnnotationVerifier buddyImplicitNullAnnotationsVerifier;
  
  private boolean inheritNullAnnotations;
  
  protected LookupEnvironment environment;
  
  public ImplicitNullAnnotationVerifier(LookupEnvironment environment, boolean inheritNullAnnotations) {
    this.buddyImplicitNullAnnotationsVerifier = this;
    this.inheritNullAnnotations = inheritNullAnnotations;
    this.environment = environment;
  }
  
  ImplicitNullAnnotationVerifier(LookupEnvironment environment) {
    CompilerOptions options = environment.globalOptions;
    this.buddyImplicitNullAnnotationsVerifier = new org.eclipse.jdt.internal.compiler.lookup.ImplicitNullAnnotationVerifier(environment, options.inheritNullAnnotations);
    this.inheritNullAnnotations = options.inheritNullAnnotations;
    this.environment = environment;
  }
  
  public void checkImplicitNullAnnotations(MethodBinding currentMethod, AbstractMethodDeclaration srcMethod, boolean complain, Scope scope) {
    try {
      ReferenceBinding currentType = currentMethod.declaringClass;
      if (currentType.id == 1)
        return; 
      boolean usesTypeAnnotations = scope.environment().usesNullTypeAnnotations();
      boolean needToApplyReturnNonNullDefault = 
        currentMethod.hasNonNullDefaultFor(16, usesTypeAnnotations);
      boolean needToApplyParameterNonNullDefault = 
        currentMethod.hasNonNullDefaultFor(8, usesTypeAnnotations);
      boolean needToApplyNonNullDefault = needToApplyReturnNonNullDefault | needToApplyParameterNonNullDefault;
      boolean isInstanceMethod = (!currentMethod.isConstructor() && !currentMethod.isStatic());
      complain &= isInstanceMethod;
      if (!needToApplyNonNullDefault && 
        !complain && (
        !this.inheritNullAnnotations || !isInstanceMethod))
        return; 
      if (isInstanceMethod) {
        List<MethodBinding> superMethodList = new ArrayList();
        if (currentType instanceof SourceTypeBinding && !currentType.isHierarchyConnected() && !currentType.isAnonymousType())
          ((SourceTypeBinding)currentType).scope.connectTypeHierarchy(); 
        int paramLen = currentMethod.parameters.length;
        findAllOverriddenMethods(currentMethod.original(), currentMethod.selector, paramLen, 
            currentType, new HashSet(), superMethodList);
        InheritedNonNullnessInfo[] inheritedNonNullnessInfos = new InheritedNonNullnessInfo[paramLen + 1];
        for (int i = 0; i < paramLen + 1; ) {
          inheritedNonNullnessInfos[i] = new InheritedNonNullnessInfo();
          i++;
        } 
        int length = superMethodList.size();
        for (int j = length; --j >= 0; ) {
          MethodBinding currentSuper = superMethodList.get(j);
          if ((currentSuper.tagBits & 0x1000L) == 0L)
            checkImplicitNullAnnotations(currentSuper, null, false, scope); 
          checkNullSpecInheritance(currentMethod, srcMethod, needToApplyReturnNonNullDefault, needToApplyParameterNonNullDefault, complain, currentSuper, null, scope, inheritedNonNullnessInfos);
          needToApplyNonNullDefault = false;
        } 
        InheritedNonNullnessInfo info = inheritedNonNullnessInfos[0];
        if (!info.complained) {
          long tagBits = 0L;
          if (info.inheritedNonNullness == Boolean.TRUE) {
            tagBits = 72057594037927936L;
          } else if (info.inheritedNonNullness == Boolean.FALSE) {
            tagBits = 36028797018963968L;
          } 
          if (tagBits != 0L)
            if (!usesTypeAnnotations) {
              currentMethod.tagBits |= tagBits;
            } else if (!currentMethod.returnType.isBaseType()) {
              LookupEnvironment env = scope.environment();
              currentMethod.returnType = env.createAnnotatedType(currentMethod.returnType, env.nullAnnotationsFromTagBits(tagBits));
            }  
        } 
        for (int k = 0; k < paramLen; k++) {
          info = inheritedNonNullnessInfos[k + 1];
          if (!info.complained && info.inheritedNonNullness != null) {
            Argument currentArg = (srcMethod == null) ? null : srcMethod.arguments[k];
            if (!usesTypeAnnotations) {
              recordArgNonNullness(currentMethod, paramLen, k, currentArg, info.inheritedNonNullness);
            } else {
              recordArgNonNullness18(currentMethod, k, currentArg, info.inheritedNonNullness, scope.environment());
            } 
          } 
        } 
      } 
      if (needToApplyNonNullDefault)
        if (!usesTypeAnnotations) {
          currentMethod.fillInDefaultNonNullness(srcMethod);
        } else {
          currentMethod.fillInDefaultNonNullness18(srcMethod, scope.environment());
        }  
    } finally {
      currentMethod.tagBits |= 0x1000L;
    } 
  }
  
  private void findAllOverriddenMethods(MethodBinding original, char[] selector, int suggestedParameterLength, ReferenceBinding currentType, Set<TypeBinding> ifcsSeen, List result) {
    if (currentType.id == 1)
      return; 
    ReferenceBinding superclass = currentType.superclass();
    if (superclass == null)
      return; 
    collectOverriddenMethods(original, selector, suggestedParameterLength, superclass, ifcsSeen, result);
    ReferenceBinding[] superInterfaces = currentType.superInterfaces();
    int ifcLen = superInterfaces.length;
    for (int i = 0; i < ifcLen; i++) {
      ReferenceBinding currentIfc = superInterfaces[i];
      if (ifcsSeen.add(currentIfc.original()))
        collectOverriddenMethods(original, selector, suggestedParameterLength, currentIfc, ifcsSeen, result); 
    } 
  }
  
  private void collectOverriddenMethods(MethodBinding original, char[] selector, int suggestedParameterLength, ReferenceBinding superType, Set ifcsSeen, List<MethodBinding> result) {
    MethodBinding[] ifcMethods = superType.getMethods(selector, suggestedParameterLength);
    int length = ifcMethods.length;
    boolean added = false;
    for (int i = 0; i < length; i++) {
      MethodBinding currentMethod = ifcMethods[i];
      if (!currentMethod.isStatic())
        if (MethodVerifier.doesMethodOverride(original, currentMethod, this.environment)) {
          result.add(currentMethod);
          added = true;
        }  
    } 
    if (!added)
      findAllOverriddenMethods(original, selector, suggestedParameterLength, superType, ifcsSeen, result); 
  }
  
  void checkNullSpecInheritance(MethodBinding currentMethod, AbstractMethodDeclaration srcMethod, boolean hasReturnNonNullDefault, boolean hasParameterNonNullDefault, boolean shouldComplain, MethodBinding inheritedMethod, MethodBinding[] allInheritedMethods, Scope scope, InheritedNonNullnessInfo[] inheritedNonNullnessInfos) {
    // Byte code:
    //   0: aload #6
    //   2: getfield tagBits : J
    //   5: ldc2_w 4096
    //   8: land
    //   9: lconst_0
    //   10: lcmp
    //   11: ifne -> 27
    //   14: aload_0
    //   15: getfield buddyImplicitNullAnnotationsVerifier : Lorg/eclipse/jdt/internal/compiler/lookup/ImplicitNullAnnotationVerifier;
    //   18: aload #6
    //   20: aconst_null
    //   21: iconst_0
    //   22: aload #8
    //   24: invokevirtual checkImplicitNullAnnotations : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/ast/AbstractMethodDeclaration;ZLorg/eclipse/jdt/internal/compiler/lookup/Scope;)V
    //   27: aload_0
    //   28: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   31: invokevirtual usesNullTypeAnnotations : ()Z
    //   34: istore #10
    //   36: aload_0
    //   37: aload #6
    //   39: iload #10
    //   41: invokespecial getReturnTypeNullnessTagBits : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)J
    //   44: lstore #11
    //   46: aload_0
    //   47: aload_1
    //   48: iload #10
    //   50: invokespecial getReturnTypeNullnessTagBits : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)J
    //   53: lstore #13
    //   55: aload_0
    //   56: getfield inheritNullAnnotations : Z
    //   59: istore #15
    //   61: aload_1
    //   62: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   65: ifnull -> 408
    //   68: aload_1
    //   69: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   72: invokevirtual isBaseType : ()Z
    //   75: ifeq -> 81
    //   78: goto -> 408
    //   81: lload #13
    //   83: lconst_0
    //   84: lcmp
    //   85: ifne -> 225
    //   88: iload #15
    //   90: ifeq -> 194
    //   93: lload #11
    //   95: lconst_0
    //   96: lcmp
    //   97: ifeq -> 194
    //   100: iload_3
    //   101: ifeq -> 136
    //   104: iload #5
    //   106: ifeq -> 136
    //   109: lload #11
    //   111: ldc2_w 36028797018963968
    //   114: lcmp
    //   115: ifne -> 136
    //   118: aload #8
    //   120: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   123: aload_1
    //   124: aload_2
    //   125: checkcast org/eclipse/jdt/internal/compiler/ast/MethodDeclaration
    //   128: getfield returnType : Lorg/eclipse/jdt/internal/compiler/ast/TypeReference;
    //   131: aload #6
    //   133: invokevirtual conflictingNullAnnotations : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/ast/ASTNode;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;)V
    //   136: aload #9
    //   138: ifnull -> 184
    //   141: aload_2
    //   142: ifnull -> 184
    //   145: aload_0
    //   146: aload #8
    //   148: aload_2
    //   149: checkcast org/eclipse/jdt/internal/compiler/ast/MethodDeclaration
    //   152: getfield returnType : Lorg/eclipse/jdt/internal/compiler/ast/TypeReference;
    //   155: aload #6
    //   157: lload #11
    //   159: ldc2_w 72057594037927936
    //   162: lcmp
    //   163: ifne -> 170
    //   166: iconst_1
    //   167: goto -> 171
    //   170: iconst_0
    //   171: invokestatic valueOf : (Z)Ljava/lang/Boolean;
    //   174: aload #9
    //   176: iconst_0
    //   177: aaload
    //   178: invokevirtual recordDeferredInheritedNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/ast/ASTNode;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Ljava/lang/Boolean;Lorg/eclipse/jdt/internal/compiler/lookup/ImplicitNullAnnotationVerifier$InheritedNonNullnessInfo;)V
    //   181: goto -> 408
    //   184: aload_0
    //   185: aload_1
    //   186: lload #11
    //   188: invokevirtual applyReturnNullBits : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;J)V
    //   191: goto -> 408
    //   194: iload_3
    //   195: ifeq -> 225
    //   198: iload #10
    //   200: ifeq -> 213
    //   203: aload_1
    //   204: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   207: invokevirtual acceptsNonNullDefault : ()Z
    //   210: ifeq -> 225
    //   213: ldc2_w 72057594037927936
    //   216: lstore #13
    //   218: aload_0
    //   219: aload_1
    //   220: lload #13
    //   222: invokevirtual applyReturnNullBits : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;J)V
    //   225: iload #5
    //   227: ifeq -> 408
    //   230: lload #11
    //   232: ldc2_w 72057594037927936
    //   235: land
    //   236: lconst_0
    //   237: lcmp
    //   238: ifeq -> 289
    //   241: lload #13
    //   243: ldc2_w 72057594037927936
    //   246: lcmp
    //   247: ifeq -> 289
    //   250: aload_2
    //   251: ifnull -> 275
    //   254: aload #8
    //   256: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   259: aload_2
    //   260: aload #6
    //   262: aload_0
    //   263: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   266: invokevirtual getNonNullAnnotationName : ()[[C
    //   269: invokevirtual illegalReturnRedefinition : (Lorg/eclipse/jdt/internal/compiler/ast/AbstractMethodDeclaration;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;[[C)V
    //   272: goto -> 408
    //   275: aload #8
    //   277: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   280: aload_1
    //   281: aload #6
    //   283: iload #10
    //   285: invokevirtual cannotImplementIncompatibleNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)V
    //   288: return
    //   289: iload #10
    //   291: ifeq -> 408
    //   294: aconst_null
    //   295: astore #16
    //   297: aload #6
    //   299: invokevirtual original : ()Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   302: getfield typeVariables : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   305: astore #17
    //   307: aload #17
    //   309: ifnull -> 343
    //   312: aload_1
    //   313: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   316: getfield id : I
    //   319: bipush #6
    //   321: if_icmpeq -> 343
    //   324: aload_0
    //   325: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   328: aload_1
    //   329: aload #17
    //   331: invokevirtual createParameterizedGenericMethod : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;[Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Lorg/eclipse/jdt/internal/compiler/lookup/ParameterizedGenericMethodBinding;
    //   334: astore #18
    //   336: aload #18
    //   338: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   341: astore #16
    //   343: aload #6
    //   345: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   348: aload_1
    //   349: getfield returnType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   352: aload #16
    //   354: aconst_null
    //   355: iconst_0
    //   356: aconst_null
    //   357: getstatic org/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode.OVERRIDE_RETURN : Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;
    //   360: invokestatic analyse : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/Substitution;ILorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;)Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching;
    //   363: invokevirtual isAnyMismatch : ()Z
    //   366: ifeq -> 408
    //   369: aload_2
    //   370: ifnull -> 394
    //   373: aload #8
    //   375: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   378: aload_2
    //   379: aload #6
    //   381: aload_0
    //   382: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   385: invokevirtual getNonNullAnnotationName : ()[[C
    //   388: invokevirtual illegalReturnRedefinition : (Lorg/eclipse/jdt/internal/compiler/ast/AbstractMethodDeclaration;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;[[C)V
    //   391: goto -> 407
    //   394: aload #8
    //   396: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   399: aload_1
    //   400: aload #6
    //   402: iload #10
    //   404: invokevirtual cannotImplementIncompatibleNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)V
    //   407: return
    //   408: aconst_null
    //   409: astore #16
    //   411: iload #5
    //   413: ifeq -> 453
    //   416: aload_1
    //   417: invokevirtual original : ()Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   420: getfield typeVariables : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   423: astore #17
    //   425: aload #17
    //   427: getstatic org/eclipse/jdt/internal/compiler/lookup/Binding.NO_TYPE_VARIABLES : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   430: if_acmpeq -> 453
    //   433: aload_0
    //   434: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   437: aload #6
    //   439: aload #17
    //   441: invokevirtual createParameterizedGenericMethod : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;[Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Lorg/eclipse/jdt/internal/compiler/lookup/ParameterizedGenericMethodBinding;
    //   444: astore #18
    //   446: aload #18
    //   448: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   451: astore #16
    //   453: aload_2
    //   454: ifnonnull -> 461
    //   457: aconst_null
    //   458: goto -> 465
    //   461: aload_2
    //   462: getfield arguments : [Lorg/eclipse/jdt/internal/compiler/ast/Argument;
    //   465: astore #17
    //   467: iconst_0
    //   468: istore #18
    //   470: aload #17
    //   472: ifnull -> 480
    //   475: aload #17
    //   477: arraylength
    //   478: istore #18
    //   480: iload #10
    //   482: ifeq -> 495
    //   485: aload_1
    //   486: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   489: arraylength
    //   490: istore #18
    //   492: goto -> 528
    //   495: aload #6
    //   497: getfield parameterNonNullness : [Ljava/lang/Boolean;
    //   500: ifnull -> 514
    //   503: aload #6
    //   505: getfield parameterNonNullness : [Ljava/lang/Boolean;
    //   508: arraylength
    //   509: istore #18
    //   511: goto -> 528
    //   514: aload_1
    //   515: getfield parameterNonNullness : [Ljava/lang/Boolean;
    //   518: ifnull -> 528
    //   521: aload_1
    //   522: getfield parameterNonNullness : [Ljava/lang/Boolean;
    //   525: arraylength
    //   526: istore #18
    //   528: iconst_0
    //   529: istore #19
    //   531: goto -> 1138
    //   534: aload_1
    //   535: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   538: iload #19
    //   540: aaload
    //   541: invokevirtual isBaseType : ()Z
    //   544: ifeq -> 550
    //   547: goto -> 1135
    //   550: aload #17
    //   552: ifnonnull -> 559
    //   555: aconst_null
    //   556: goto -> 564
    //   559: aload #17
    //   561: iload #19
    //   563: aaload
    //   564: astore #20
    //   566: aload_0
    //   567: aload #6
    //   569: iload #19
    //   571: iload #10
    //   573: invokespecial getParameterNonNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;IZ)Ljava/lang/Boolean;
    //   576: astore #21
    //   578: aload_0
    //   579: aload_1
    //   580: iload #19
    //   582: iload #10
    //   584: invokespecial getParameterNonNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;IZ)Ljava/lang/Boolean;
    //   587: astore #22
    //   589: aload #22
    //   591: ifnonnull -> 785
    //   594: aload #21
    //   596: ifnull -> 718
    //   599: iload #15
    //   601: ifeq -> 718
    //   604: iload #4
    //   606: ifeq -> 640
    //   609: iload #5
    //   611: ifeq -> 640
    //   614: aload #21
    //   616: getstatic java/lang/Boolean.FALSE : Ljava/lang/Boolean;
    //   619: if_acmpne -> 640
    //   622: aload #20
    //   624: ifnull -> 640
    //   627: aload #8
    //   629: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   632: aload_1
    //   633: aload #20
    //   635: aload #6
    //   637: invokevirtual conflictingNullAnnotations : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/ast/ASTNode;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;)V
    //   640: aload #9
    //   642: ifnull -> 679
    //   645: aload_2
    //   646: ifnull -> 679
    //   649: aload_0
    //   650: aload #8
    //   652: aload_2
    //   653: getfield arguments : [Lorg/eclipse/jdt/internal/compiler/ast/Argument;
    //   656: iload #19
    //   658: aaload
    //   659: getfield type : Lorg/eclipse/jdt/internal/compiler/ast/TypeReference;
    //   662: aload #6
    //   664: aload #21
    //   666: aload #9
    //   668: iload #19
    //   670: iconst_1
    //   671: iadd
    //   672: aaload
    //   673: invokevirtual recordDeferredInheritedNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/ast/ASTNode;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Ljava/lang/Boolean;Lorg/eclipse/jdt/internal/compiler/lookup/ImplicitNullAnnotationVerifier$InheritedNonNullnessInfo;)V
    //   676: goto -> 1135
    //   679: iload #10
    //   681: ifne -> 700
    //   684: aload_0
    //   685: aload_1
    //   686: iload #18
    //   688: iload #19
    //   690: aload #20
    //   692: aload #21
    //   694: invokevirtual recordArgNonNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;IILorg/eclipse/jdt/internal/compiler/ast/Argument;Ljava/lang/Boolean;)V
    //   697: goto -> 1135
    //   700: aload_0
    //   701: aload_1
    //   702: iload #19
    //   704: aload #20
    //   706: aload #21
    //   708: aload_0
    //   709: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   712: invokevirtual recordArgNonNullness18 : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;ILorg/eclipse/jdt/internal/compiler/ast/Argument;Ljava/lang/Boolean;Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;)V
    //   715: goto -> 1135
    //   718: iload #4
    //   720: ifeq -> 785
    //   723: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   726: astore #22
    //   728: iload #10
    //   730: ifne -> 750
    //   733: aload_0
    //   734: aload_1
    //   735: iload #18
    //   737: iload #19
    //   739: aload #20
    //   741: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   744: invokevirtual recordArgNonNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;IILorg/eclipse/jdt/internal/compiler/ast/Argument;Ljava/lang/Boolean;)V
    //   747: goto -> 785
    //   750: aload_1
    //   751: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   754: iload #19
    //   756: aaload
    //   757: invokevirtual acceptsNonNullDefault : ()Z
    //   760: ifeq -> 782
    //   763: aload_0
    //   764: aload_1
    //   765: iload #19
    //   767: aload #20
    //   769: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   772: aload_0
    //   773: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   776: invokevirtual recordArgNonNullness18 : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;ILorg/eclipse/jdt/internal/compiler/ast/Argument;Ljava/lang/Boolean;Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;)V
    //   779: goto -> 785
    //   782: aconst_null
    //   783: astore #22
    //   785: iload #5
    //   787: ifeq -> 1135
    //   790: aload #21
    //   792: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   795: if_acmpne -> 810
    //   798: aload_0
    //   799: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   802: invokevirtual getNonNullAnnotationName : ()[[C
    //   805: astore #23
    //   807: goto -> 819
    //   810: aload_0
    //   811: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   814: invokevirtual getNullableAnnotationName : ()[[C
    //   817: astore #23
    //   819: aload #21
    //   821: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   824: if_acmpeq -> 889
    //   827: aload #22
    //   829: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   832: if_acmpne -> 889
    //   835: aload #20
    //   837: ifnull -> 874
    //   840: aload #8
    //   842: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   845: aload #20
    //   847: aload #6
    //   849: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   852: aload #21
    //   854: ifnonnull -> 861
    //   857: aconst_null
    //   858: goto -> 868
    //   861: aload_0
    //   862: getfield environment : Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   865: invokevirtual getNullableAnnotationName : ()[[C
    //   868: invokevirtual illegalRedefinitionToNonNullParameter : (Lorg/eclipse/jdt/internal/compiler/ast/Argument;Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;[[C)V
    //   871: goto -> 1135
    //   874: aload #8
    //   876: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   879: aload_1
    //   880: aload #6
    //   882: iconst_0
    //   883: invokevirtual cannotImplementIncompatibleNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)V
    //   886: goto -> 1135
    //   889: aload #22
    //   891: ifnonnull -> 1041
    //   894: aload #21
    //   896: getstatic java/lang/Boolean.FALSE : Ljava/lang/Boolean;
    //   899: if_acmpne -> 942
    //   902: aload #20
    //   904: ifnull -> 927
    //   907: aload #8
    //   909: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   912: aload #20
    //   914: aload #6
    //   916: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   919: aload #23
    //   921: invokevirtual parameterLackingNullableAnnotation : (Lorg/eclipse/jdt/internal/compiler/ast/Argument;Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;[[C)V
    //   924: goto -> 1135
    //   927: aload #8
    //   929: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   932: aload_1
    //   933: aload #6
    //   935: iconst_0
    //   936: invokevirtual cannotImplementIncompatibleNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)V
    //   939: goto -> 1135
    //   942: aload #21
    //   944: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   947: if_acmpne -> 1041
    //   950: aload #7
    //   952: ifnull -> 1021
    //   955: aload #7
    //   957: dup
    //   958: astore #27
    //   960: arraylength
    //   961: istore #26
    //   963: iconst_0
    //   964: istore #25
    //   966: goto -> 1014
    //   969: aload #27
    //   971: iload #25
    //   973: aaload
    //   974: astore #24
    //   976: aload #6
    //   978: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   981: aload #24
    //   983: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   986: invokestatic equalsEquals : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   989: ifeq -> 1011
    //   992: aload_0
    //   993: aload #24
    //   995: iload #19
    //   997: iload #10
    //   999: invokespecial getParameterNonNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;IZ)Ljava/lang/Boolean;
    //   1002: getstatic java/lang/Boolean.TRUE : Ljava/lang/Boolean;
    //   1005: if_acmpeq -> 1011
    //   1008: goto -> 1135
    //   1011: iinc #25, 1
    //   1014: iload #25
    //   1016: iload #26
    //   1018: if_icmplt -> 969
    //   1021: aload #8
    //   1023: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1026: aload #20
    //   1028: aload #6
    //   1030: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   1033: aload #23
    //   1035: invokevirtual parameterLackingNonnullAnnotation : (Lorg/eclipse/jdt/internal/compiler/ast/Argument;Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;[[C)V
    //   1038: goto -> 1135
    //   1041: iload #10
    //   1043: ifeq -> 1135
    //   1046: aload #6
    //   1048: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   1051: iload #19
    //   1053: aaload
    //   1054: astore #24
    //   1056: aload #16
    //   1058: ifnull -> 1069
    //   1061: aload #16
    //   1063: iload #19
    //   1065: aaload
    //   1066: goto -> 1070
    //   1069: aconst_null
    //   1070: astore #25
    //   1072: aload_1
    //   1073: getfield parameters : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   1076: iload #19
    //   1078: aaload
    //   1079: aload #24
    //   1081: aload #25
    //   1083: aconst_null
    //   1084: iconst_0
    //   1085: aconst_null
    //   1086: getstatic org/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode.OVERRIDE : Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;
    //   1089: invokestatic analyse : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/Substitution;ILorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;)Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching;
    //   1092: invokevirtual isAnyMismatch : ()Z
    //   1095: ifeq -> 1135
    //   1098: aload #20
    //   1100: ifnull -> 1123
    //   1103: aload #8
    //   1105: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1108: aload #20
    //   1110: aload #6
    //   1112: getfield declaringClass : Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   1115: aload #24
    //   1117: invokevirtual illegalParameterRedefinition : (Lorg/eclipse/jdt/internal/compiler/ast/Argument;Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   1120: goto -> 1135
    //   1123: aload #8
    //   1125: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1128: aload_1
    //   1129: aload #6
    //   1131: iconst_0
    //   1132: invokevirtual cannotImplementIncompatibleNullness : (Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;Z)V
    //   1135: iinc #19, 1
    //   1138: iload #19
    //   1140: iload #18
    //   1142: if_icmplt -> 534
    //   1145: iload #5
    //   1147: ifeq -> 1255
    //   1150: iload #10
    //   1152: ifeq -> 1255
    //   1155: aload_2
    //   1156: ifnull -> 1255
    //   1159: aload_1
    //   1160: invokevirtual typeVariables : ()[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   1163: astore #19
    //   1165: aload #6
    //   1167: invokevirtual typeVariables : ()[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   1170: astore #20
    //   1172: aload #19
    //   1174: getstatic org/eclipse/jdt/internal/compiler/lookup/Binding.NO_TYPE_VARIABLES : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   1177: if_acmpeq -> 1255
    //   1180: aload #19
    //   1182: arraylength
    //   1183: aload #20
    //   1185: arraylength
    //   1186: if_icmpne -> 1255
    //   1189: iconst_0
    //   1190: istore #21
    //   1192: goto -> 1247
    //   1195: aload #20
    //   1197: iload #21
    //   1199: aaload
    //   1200: astore #22
    //   1202: aload #22
    //   1204: aload #19
    //   1206: iload #21
    //   1208: aaload
    //   1209: aconst_null
    //   1210: aconst_null
    //   1211: iconst_m1
    //   1212: aconst_null
    //   1213: getstatic org/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode.BOUND_CHECK : Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;
    //   1216: invokestatic analyse : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/Substitution;ILorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching$CheckMode;)Lorg/eclipse/jdt/internal/compiler/ast/NullAnnotationMatching;
    //   1219: invokevirtual isAnyMismatch : ()Z
    //   1222: ifeq -> 1244
    //   1225: aload #8
    //   1227: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1230: aload #22
    //   1232: aload #6
    //   1234: aload_2
    //   1235: invokevirtual typeParameters : ()[Lorg/eclipse/jdt/internal/compiler/ast/TypeParameter;
    //   1238: iload #21
    //   1240: aaload
    //   1241: invokevirtual cannotRedefineTypeArgumentNullity : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/Binding;Lorg/eclipse/jdt/internal/compiler/ast/ASTNode;)V
    //   1244: iinc #21, 1
    //   1247: iload #21
    //   1249: aload #19
    //   1251: arraylength
    //   1252: if_icmplt -> 1195
    //   1255: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #238	-> 0
    //   #240	-> 14
    //   #242	-> 27
    //   #243	-> 36
    //   #244	-> 46
    //   #246	-> 55
    //   #250	-> 61
    //   #251	-> 78
    //   #252	-> 81
    //   #254	-> 88
    //   #255	-> 93
    //   #256	-> 100
    //   #258	-> 104
    //   #259	-> 118
    //   #262	-> 136
    //   #263	-> 145
    //   #264	-> 155
    //   #263	-> 178
    //   #265	-> 181
    //   #267	-> 184
    //   #269	-> 191
    //   #272	-> 194
    //   #273	-> 213
    //   #274	-> 218
    //   #277	-> 225
    //   #278	-> 230
    //   #279	-> 241
    //   #281	-> 250
    //   #282	-> 254
    //   #283	-> 262
    //   #282	-> 269
    //   #284	-> 272
    //   #286	-> 275
    //   #287	-> 288
    //   #290	-> 289
    //   #291	-> 294
    //   #292	-> 297
    //   #293	-> 307
    //   #294	-> 324
    //   #295	-> 336
    //   #297	-> 343
    //   #298	-> 369
    //   #299	-> 373
    //   #300	-> 381
    //   #299	-> 388
    //   #300	-> 391
    //   #302	-> 394
    //   #303	-> 407
    //   #310	-> 408
    //   #311	-> 411
    //   #312	-> 416
    //   #313	-> 425
    //   #314	-> 433
    //   #315	-> 446
    //   #319	-> 453
    //   #321	-> 467
    //   #322	-> 470
    //   #323	-> 475
    //   #324	-> 480
    //   #325	-> 485
    //   #326	-> 495
    //   #327	-> 503
    //   #328	-> 514
    //   #329	-> 521
    //   #332	-> 528
    //   #333	-> 534
    //   #335	-> 550
    //   #336	-> 555
    //   #335	-> 564
    //   #337	-> 566
    //   #338	-> 578
    //   #340	-> 589
    //   #342	-> 594
    //   #343	-> 599
    //   #344	-> 604
    //   #346	-> 609
    //   #347	-> 614
    //   #348	-> 622
    //   #350	-> 627
    //   #354	-> 640
    //   #355	-> 649
    //   #356	-> 662
    //   #355	-> 673
    //   #357	-> 676
    //   #359	-> 679
    //   #360	-> 684
    //   #362	-> 700
    //   #364	-> 715
    //   #367	-> 718
    //   #368	-> 723
    //   #369	-> 728
    //   #370	-> 733
    //   #371	-> 750
    //   #372	-> 763
    //   #374	-> 782
    //   #377	-> 785
    //   #379	-> 790
    //   #380	-> 798
    //   #381	-> 807
    //   #382	-> 810
    //   #384	-> 819
    //   #385	-> 827
    //   #388	-> 835
    //   #389	-> 840
    //   #390	-> 845
    //   #391	-> 847
    //   #392	-> 852
    //   #389	-> 868
    //   #393	-> 871
    //   #394	-> 874
    //   #396	-> 886
    //   #397	-> 889
    //   #400	-> 894
    //   #401	-> 902
    //   #402	-> 907
    //   #403	-> 912
    //   #404	-> 914
    //   #405	-> 919
    //   #402	-> 921
    //   #406	-> 924
    //   #407	-> 927
    //   #409	-> 939
    //   #410	-> 942
    //   #412	-> 950
    //   #414	-> 955
    //   #415	-> 976
    //   #416	-> 1008
    //   #414	-> 1011
    //   #418	-> 1021
    //   #419	-> 1026
    //   #420	-> 1028
    //   #421	-> 1033
    //   #418	-> 1035
    //   #422	-> 1038
    //   #425	-> 1041
    //   #426	-> 1046
    //   #427	-> 1056
    //   #428	-> 1072
    //   #429	-> 1098
    //   #430	-> 1103
    //   #432	-> 1123
    //   #332	-> 1135
    //   #438	-> 1145
    //   #439	-> 1159
    //   #440	-> 1165
    //   #441	-> 1172
    //   #442	-> 1189
    //   #443	-> 1195
    //   #444	-> 1202
    //   #445	-> 1225
    //   #442	-> 1244
    //   #449	-> 1255
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	1256	0	this	Lorg/eclipse/jdt/internal/compiler/lookup/ImplicitNullAnnotationVerifier;
    //   0	1256	1	currentMethod	Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   0	1256	2	srcMethod	Lorg/eclipse/jdt/internal/compiler/ast/AbstractMethodDeclaration;
    //   0	1256	3	hasReturnNonNullDefault	Z
    //   0	1256	4	hasParameterNonNullDefault	Z
    //   0	1256	5	shouldComplain	Z
    //   0	1256	6	inheritedMethod	Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   0	1256	7	allInheritedMethods	[Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   0	1256	8	scope	Lorg/eclipse/jdt/internal/compiler/lookup/Scope;
    //   0	1256	9	inheritedNonNullnessInfos	[Lorg/eclipse/jdt/internal/compiler/lookup/ImplicitNullAnnotationVerifier$InheritedNonNullnessInfo;
    //   36	1220	10	useTypeAnnotations	Z
    //   46	1210	11	inheritedNullnessBits	J
    //   55	1201	13	currentNullnessBits	J
    //   61	1195	15	shouldInherit	Z
    //   297	111	16	substituteReturnType	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   307	101	17	typeVariables	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   336	7	18	substitute	Lorg/eclipse/jdt/internal/compiler/lookup/ParameterizedGenericMethodBinding;
    //   411	845	16	substituteParameters	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   425	28	17	typeVariables	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   446	7	18	substitute	Lorg/eclipse/jdt/internal/compiler/lookup/ParameterizedGenericMethodBinding;
    //   467	789	17	currentArguments	[Lorg/eclipse/jdt/internal/compiler/ast/Argument;
    //   470	786	18	length	I
    //   531	614	19	i	I
    //   566	569	20	currentArgument	Lorg/eclipse/jdt/internal/compiler/ast/Argument;
    //   578	557	21	inheritedNonNullNess	Ljava/lang/Boolean;
    //   589	546	22	currentNonNullNess	Ljava/lang/Boolean;
    //   807	3	23	annotationName	[[C
    //   819	316	23	annotationName	[[C
    //   976	35	24	one	Lorg/eclipse/jdt/internal/compiler/lookup/MethodBinding;
    //   1056	79	24	inheritedParameter	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   1072	63	25	substituteParameter	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   1165	90	19	currentTypeVariables	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   1172	83	20	inheritedTypeVariables	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   1192	63	21	i	I
    //   1202	42	22	inheritedVariable	Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
  }
  
  void applyReturnNullBits(MethodBinding method, long nullnessBits) {
    if (this.environment.usesNullTypeAnnotations()) {
      if (!method.returnType.isBaseType())
        method.returnType = this.environment.createAnnotatedType(method.returnType, this.environment.nullAnnotationsFromTagBits(nullnessBits)); 
    } else {
      method.tagBits |= nullnessBits;
    } 
  }
  
  private Boolean getParameterNonNullness(MethodBinding method, int i, boolean useTypeAnnotations) {
    if (useTypeAnnotations) {
      TypeBinding parameter = method.parameters[i];
      if (parameter != null) {
        long nullBits = NullAnnotationMatching.validNullTagBits(parameter.tagBits);
        if (nullBits != 0L)
          return Boolean.valueOf((nullBits == 72057594037927936L)); 
      } 
      return null;
    } 
    return (method.parameterNonNullness == null) ? 
      null : method.parameterNonNullness[i];
  }
  
  private long getReturnTypeNullnessTagBits(MethodBinding method, boolean useTypeAnnotations) {
    if (useTypeAnnotations) {
      if (method.returnType == null)
        return 0L; 
      return NullAnnotationMatching.validNullTagBits(method.returnType.tagBits);
    } 
    return method.tagBits & 0x180000000000000L;
  }
  
  protected void recordDeferredInheritedNullness(Scope scope, ASTNode location, MethodBinding inheritedMethod, Boolean inheritedNonNullness, InheritedNonNullnessInfo nullnessInfo) {
    if (nullnessInfo.inheritedNonNullness != null && nullnessInfo.inheritedNonNullness != inheritedNonNullness) {
      scope.problemReporter().conflictingInheritedNullAnnotations(location, 
          nullnessInfo.inheritedNonNullness.booleanValue(), nullnessInfo.annotationOrigin, 
          inheritedNonNullness.booleanValue(), inheritedMethod);
      nullnessInfo.complained = true;
    } else {
      nullnessInfo.inheritedNonNullness = inheritedNonNullness;
      nullnessInfo.annotationOrigin = inheritedMethod;
    } 
  }
  
  void recordArgNonNullness(MethodBinding method, int paramCount, int paramIdx, Argument currentArgument, Boolean nonNullNess) {
    if (method.parameterNonNullness == null)
      method.parameterNonNullness = new Boolean[paramCount]; 
    method.parameterNonNullness[paramIdx] = nonNullNess;
    if (currentArgument != null)
      currentArgument.binding.tagBits |= nonNullNess.booleanValue() ? 
        72057594037927936L : 36028797018963968L; 
  }
  
  void recordArgNonNullness18(MethodBinding method, int paramIdx, Argument currentArgument, Boolean nonNullNess, LookupEnvironment env) {
    AnnotationBinding annotationBinding = nonNullNess.booleanValue() ? env.getNonNullAnnotation() : env.getNullableAnnotation();
    method.parameters[paramIdx] = env.createAnnotatedType(method.parameters[paramIdx], new AnnotationBinding[] { annotationBinding });
    if (currentArgument != null)
      currentArgument.binding.type = method.parameters[paramIdx]; 
  }
  
  static boolean areParametersEqual(MethodBinding one, MethodBinding two) {
    TypeBinding[] oneArgs = one.parameters;
    TypeBinding[] twoArgs = two.parameters;
    if (oneArgs == twoArgs)
      return true; 
    int length = oneArgs.length;
    if (length != twoArgs.length)
      return false; 
    int i;
    for (i = 0; i < length; i++) {
      if (!areTypesEqual(oneArgs[i], twoArgs[i])) {
        if (oneArgs[i].leafComponentType().isRawType() && 
          oneArgs[i].dimensions() == twoArgs[i].dimensions() && oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType())) {
          if (one.typeVariables != Binding.NO_TYPE_VARIABLES)
            return false; 
          for (int j = 0; j < i; j++) {
            if (oneArgs[j].leafComponentType().isParameterizedTypeWithActualArguments())
              return false; 
          } 
          break;
        } 
        return false;
      } 
    } 
    for (; ++i < length; i++) {
      if (!areTypesEqual(oneArgs[i], twoArgs[i])) {
        if (!oneArgs[i].leafComponentType().isRawType() || 
          oneArgs[i].dimensions() != twoArgs[i].dimensions() || !oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType()))
          return false; 
      } else if (oneArgs[i].leafComponentType().isParameterizedTypeWithActualArguments()) {
        return false;
      } 
    } 
    return true;
  }
  
  static boolean areTypesEqual(TypeBinding one, TypeBinding two) {
    if (TypeBinding.equalsEquals(one, two))
      return true; 
    switch (one.kind()) {
      case 4:
        switch (two.kind()) {
          case 260:
          case 1028:
            if (TypeBinding.equalsEquals(one, two.erasure()))
              return true; 
            break;
        } 
        break;
      case 260:
      case 1028:
        switch (two.kind()) {
          case 4:
            if (TypeBinding.equalsEquals(one.erasure(), two))
              return true; 
            break;
        } 
        break;
    } 
    if (one.isParameterizedType() && two.isParameterizedType())
      return (one.isEquivalentTo(two) && two.isEquivalentTo(one)); 
    return false;
  }
}
